@@ -38,6 +38,7 @@
 #include <glib/gstdio.h>
 
 #include "camel-groupwise-folder.h"
+#include "camel-groupwise-settings.h"
 #include "camel-groupwise-store-summary.h"
 #include "camel-groupwise-store.h"
 #include "camel-groupwise-summary.h"
@@ -58,19 +59,17 @@ const gchar * CREATE_CURSOR_VIEW = "peek id default recipient threading hasAttac
 
 struct _CamelGroupwiseStorePrivate {
 	gchar *server_name;
-	gchar *port;
 	gchar *user;
-	gchar *use_ssl;
 
 	gchar *base_url;
 
-	GHashTable *id_hash; /*get names from ids*/
-	GHashTable *name_hash;/*get ids from names*/
+	GHashTable *id_hash; /*get names from ids */
+	GHashTable *name_hash;/*get ids from names */
 	GHashTable *parent_hash;
 	EGwConnection *cnc;
 };
 
-extern CamelServiceAuthType camel_groupwise_password_authtype; /*for the query_auth_types function*/
+extern CamelServiceAuthType camel_groupwise_password_authtype; /*for the query_auth_types function */
 
 static GInitableIface *parent_initable_interface;
 
@@ -91,7 +90,8 @@ groupwise_hash_folder_name (gconstpointer key)
 }
 
 static gint
-groupwise_compare_folder_name (gconstpointer a, gconstpointer b)
+groupwise_compare_folder_name (gconstpointer a,
+                               gconstpointer b)
 {
 	gconstpointer aname = a, bname = b;
 
@@ -105,20 +105,35 @@ groupwise_auth_loop (CamelService *service,
 {
 	CamelSession *session = camel_service_get_session (service);
 	CamelStore *store = CAMEL_STORE (service);
+	CamelSettings *settings;
 	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (store);
 	CamelGroupwiseStorePrivate *priv = groupwise_store->priv;
+	CamelNetworkSecurityMethod security_method;
 	CamelURL *url;
 	gboolean authenticated = FALSE;
 	gchar *uri;
 	guint32 prompt_flags = CAMEL_SESSION_PASSWORD_SECRET;
+	const gchar *scheme;
 	EGwConnectionErrors errors = {E_GW_CONNECTION_STATUS_INVALID_OBJECT, NULL};
+	guint16 soap_port;
 
 	url = camel_service_get_camel_url (service);
+	settings = camel_service_get_settings (service);
 
-	if (priv->use_ssl && !g_str_equal (priv->use_ssl, "never"))
-		uri = g_strconcat ("https://", priv->server_name, ":", priv->port, "/soap", NULL);
+	security_method = camel_network_settings_get_security_method (
+		CAMEL_NETWORK_SETTINGS (settings));
+
+	soap_port = camel_groupwise_settings_get_soap_port (
+		CAMEL_GROUPWISE_SETTINGS (settings));
+
+	if (security_method == CAMEL_NETWORK_SECURITY_METHOD_NONE)
+		scheme = "http";
 	else
-		uri = g_strconcat ("http://", priv->server_name, ":", priv->port, "/soap", NULL);
+		scheme = "https";
+
+	uri = g_strdup_printf (
+		"%s://%s:%u/soap", scheme, priv->server_name, soap_port);
+
 	url->passwd = NULL;
 
 	while (!authenticated) {
@@ -144,11 +159,7 @@ groupwise_auth_loop (CamelService *service,
 		}
 
 		priv->cnc = e_gw_connection_new_with_error_handler (uri, priv->user, url->passwd, &errors);
-		if (!E_IS_GW_CONNECTION(priv->cnc) && priv->use_ssl && g_str_equal (priv->use_ssl, "when-possible")) {
-			gchar *http_uri = g_strconcat ("http://", uri + 8, NULL);
-			priv->cnc = e_gw_connection_new (http_uri, priv->user, url->passwd);
-			g_free (http_uri);
-		}
+
 		if (!E_IS_GW_CONNECTION (priv->cnc)) {
 			if (errors.status == E_GW_CONNECTION_STATUS_INVALID_PASSWORD) {
 				/* We need to un-cache the password before prompting again */
@@ -179,16 +190,30 @@ check_for_connection (CamelService *service,
 {
 	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (service);
 	CamelGroupwiseStorePrivate *priv = groupwise_store->priv;
+	CamelSettings *settings;
 	struct addrinfo hints, *ai;
 	GError *local_error = NULL;
 
+	settings = camel_service_get_settings (service);
+
 	memset (&hints, 0, sizeof (hints));
 	hints.ai_socktype = SOCK_STREAM;
 	hints.ai_family = PF_UNSPEC;
 	ai = camel_getaddrinfo(priv->server_name, "groupwise", &hints, cancellable, &local_error);
-	if (ai == NULL && priv->port != NULL && g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+	if (ai == NULL && g_error_matches (local_error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+		gchar *port_string;
+		guint16 soap_port;
+
 		g_clear_error (&local_error);
-		ai = camel_getaddrinfo (priv->server_name, priv->port, &hints, cancellable, &local_error);
+
+		soap_port = camel_groupwise_settings_get_soap_port (
+			CAMEL_GROUPWISE_SETTINGS (settings));
+
+		port_string = g_strdup_printf ("%u", soap_port);
+		ai = camel_getaddrinfo (
+			priv->server_name, port_string, &hints,
+			cancellable, &local_error);
+		g_free (port_string);
 	}
 
 	if (ai == NULL) {
@@ -204,7 +229,8 @@ check_for_connection (CamelService *service,
 
 /* resets the current folder. To just free current folder, pass NULL for folder */
 void
-groupwise_store_set_current_folder (CamelGroupwiseStore *groupwise_store, CamelFolder *folder)
+groupwise_store_set_current_folder (CamelGroupwiseStore *groupwise_store,
+                                    CamelFolder *folder)
 {
 
 	camel_service_lock (CAMEL_SERVICE (groupwise_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
@@ -286,7 +312,9 @@ groupwise_connect_sync (CamelService *service,
 }
 #if 0
 static void
-groupwise_disconnect_cleanup (CamelService *service, gboolean clean, GError **error)
+groupwise_disconnect_cleanup (CamelService *service,
+                              gboolean clean,
+                              GError **error)
 {
 	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (service);
 	CamelGroupwiseStorePrivate *priv = groupwise_store->priv;
@@ -360,7 +388,7 @@ groupwise_disconnect_sync (CamelService *service,
 	return TRUE;
 }
 
-static  GList*
+static  GList *
 groupwise_store_query_auth_types_sync (CamelService *service,
                                        GCancellable *cancellable,
                                        GError **error)
@@ -388,7 +416,9 @@ groupwise_is_system_folder (const gchar *folder_name)
 
 /*Build/populate CamelFolderInfo structure based on the imap_build_folder_info function*/
 static CamelFolderInfo *
-groupwise_build_folder_info (CamelGroupwiseStore *gw_store, const gchar *parent_name, const gchar *folder_name)
+groupwise_build_folder_info (CamelGroupwiseStore *gw_store,
+                             const gchar *parent_name,
+                             const gchar *folder_name)
 {
 	const gchar *name;
 	CamelFolderInfo *fi;
@@ -428,7 +458,9 @@ groupwise_build_folder_info (CamelGroupwiseStore *gw_store, const gchar *parent_
 }
 
 static void
-groupwise_forget_folder (CamelGroupwiseStore *gw_store, const gchar *folder_name, GError **error)
+groupwise_forget_folder (CamelGroupwiseStore *gw_store,
+                         const gchar *folder_name,
+                         GError **error)
 {
 	CamelService *service;
 	const gchar *user_data_dir;
@@ -624,7 +656,7 @@ groupwise_store_get_folder_sync (CamelStore *store,
 						count = total;
 
 					camel_operation_progress (
-						cancellable, (100*count)/total);
+						cancellable, (100 * count) / total);
 				}
 
 				gw_update_summary (folder, list, cancellable, error);
@@ -692,7 +724,7 @@ gw_store_reload_folder (CamelGroupwiseStore *gw_store,
 	}
 
 	if (!E_IS_GW_CONNECTION ( priv->cnc)) {
-		if (!groupwise_connect_sync (CAMEL_SERVICE ((CamelStore*) gw_store), cancellable, error)) {
+		if (!groupwise_connect_sync (CAMEL_SERVICE ((CamelStore *) gw_store), cancellable, error)) {
 			camel_service_unlock (CAMEL_SERVICE (gw_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 			return FALSE;
 		}
@@ -751,7 +783,7 @@ gw_store_reload_folder (CamelGroupwiseStore *gw_store,
 							return FALSE;
 					}
 					/* This full block is repeated in the get_folder code as well. We need
-					   better modularity */
+					 * better modularity */
 					count += CURSOR_ITEM_LIMIT;
 
 					if (total > 0) {
@@ -767,8 +799,8 @@ gw_store_reload_folder (CamelGroupwiseStore *gw_store,
 					gw_update_summary (folder, list, cancellable, error);
 
 					/* For shared-folders created by the user, we don't get the total number of messages,
-					   in the getFolderList call. So, we need to wait until an empty list is returned in the
-					   read cursor call. Hence, we need the !list checking in the code below */
+					 * in the getFolderList call. So, we need to wait until an empty list is returned in the
+					 * read cursor call. Hence, we need the !list checking in the code below */
 					if (count == total || !list)
 							done = TRUE;
 
@@ -835,9 +867,9 @@ convert_to_folder_info (CamelGroupwiseStore *store,
 		(type == E_GW_CONTAINER_TYPE_TRASH) )
 		fi->flags |= CAMEL_FOLDER_SYSTEM;
 	/*
-	   parent_hash contains the "parent id <-> container id" combination. So we form
-	   the path for the full name in camelfolder info by looking up the hash table until
-	   NULL is found
+	 * parent_hash contains the "parent id <-> container id" combination. So we form
+	 * the path for the full name in camelfolder info by looking up the hash table until
+	 * NULL is found
 	 */
 
 	parent = e_gw_container_get_parent_id (container);
@@ -898,7 +930,9 @@ convert_to_folder_info (CamelGroupwiseStore *store,
 }
 
 static void
-get_folders_free (gpointer k, gpointer v, gpointer d)
+get_folders_free (gpointer k,
+                  gpointer v,
+                  gpointer d)
 {
 	CamelFolderInfo *fi = v;
 	camel_folder_info_free (fi);
@@ -929,7 +963,7 @@ groupwise_folders_sync (CamelGroupwiseStore *store,
 	list = folder_list;
 
 	/*populate the hash table for finding the mapping from container id <-> folder name*/
-	for (;temp_list != NULL; temp_list = g_list_next (temp_list) ) {
+	for (; temp_list != NULL; temp_list = g_list_next (temp_list) ) {
 		const gchar *name, *id, *parent;
 		name = e_gw_container_get_name (E_GW_CONTAINER (temp_list->data));
 		id = e_gw_container_get_id (E_GW_CONTAINER (temp_list->data));
@@ -950,7 +984,7 @@ groupwise_folders_sync (CamelGroupwiseStore *store,
 
 	present = g_hash_table_new (g_str_hash, g_str_equal);
 
-	for (;folder_list != NULL; folder_list = g_list_next (folder_list)) {
+	for (; folder_list != NULL; folder_list = g_list_next (folder_list)) {
 		EGwContainerType type;
 		EGwContainer *container = E_GW_CONTAINER (folder_list->data);
 
@@ -977,7 +1011,7 @@ groupwise_folders_sync (CamelGroupwiseStore *store,
 	count = camel_store_summary_count ((CamelStoreSummary *) store->summary);
 
 	count = camel_store_summary_count ((CamelStoreSummary *) store->summary);
-	for (i=0;i<count;i++) {
+	for (i = 0; i < count; i++) {
 		si = camel_store_summary_index ((CamelStoreSummary *) store->summary, i);
 		if (si == NULL)
 			continue;
@@ -1000,8 +1034,10 @@ groupwise_folders_sync (CamelGroupwiseStore *store,
 }
 
 static CamelFolderInfo *
-groupwise_get_folder_info_offline (CamelStore *store, const gchar *top,
-			 guint32 flags, GError **error)
+groupwise_get_folder_info_offline (CamelStore *store,
+                                   const gchar *top,
+                                   guint32 flags,
+                                   GError **error)
 {
 	CamelGroupwiseStore *groupwise_store = CAMEL_GROUPWISE_STORE (store);
 	CamelFolderInfo *fi;
@@ -1025,7 +1061,7 @@ groupwise_get_folder_info_offline (CamelStore *store, const gchar *top,
 
 	path = gw_concat (name, "*");
 
-	for (i=0;i<camel_store_summary_count ((CamelStoreSummary *) groupwise_store->summary);i++) {
+	for (i = 0; i < camel_store_summary_count ((CamelStoreSummary *) groupwise_store->summary); i++) {
 		CamelStoreInfo *si = camel_store_summary_index ((CamelStoreSummary *) groupwise_store->summary, i);
 
 		if (si == NULL)
@@ -1060,7 +1096,7 @@ groupwise_store_get_folder_info_sync (CamelStore *store,
 	CamelFolderInfo *info = NULL;
 
 	/* Do not call groupwise_store_connected function as it would internall call folders_sync
-	   to populate the hash table which is used for mapping container id */
+	 * to populate the hash table which is used for mapping container id */
 	if (!(camel_offline_store_get_online (CAMEL_OFFLINE_STORE (store))
 	    && camel_service_connect_sync ((CamelService *) store, error)))
 		goto offline;
@@ -1119,7 +1155,7 @@ create_junk_folder (CamelStore *store)
 	return root;
 }
 
-static CamelFolderInfo*
+static CamelFolderInfo *
 groupwise_store_create_folder_sync (CamelStore *store,
                                     const gchar *parent_name,
                                     const gchar *folder_name,
@@ -1301,7 +1337,8 @@ groupwise_store_rename_folder_sync (CamelStore *store,
 }
 
 gchar *
-groupwise_get_name (CamelService *service, gboolean brief)
+groupwise_get_name (CamelService *service,
+                    gboolean brief)
 {
 	CamelURL *url;
 
@@ -1317,7 +1354,8 @@ groupwise_get_name (CamelService *service, gboolean brief)
 }
 
 const gchar *
-camel_groupwise_store_container_id_lookup (CamelGroupwiseStore *gw_store, const gchar *folder_name)
+camel_groupwise_store_container_id_lookup (CamelGroupwiseStore *gw_store,
+                                           const gchar *folder_name)
 {
 	CamelGroupwiseStorePrivate *priv = gw_store->priv;
 
@@ -1325,7 +1363,8 @@ camel_groupwise_store_container_id_lookup (CamelGroupwiseStore *gw_store, const
 }
 
 const gchar *
-camel_groupwise_store_folder_lookup (CamelGroupwiseStore *gw_store, const gchar *container_id)
+camel_groupwise_store_folder_lookup (CamelGroupwiseStore *gw_store,
+                                     const gchar *container_id)
 {
 	CamelGroupwiseStorePrivate *priv = gw_store->priv;
 
@@ -1375,18 +1414,22 @@ groupwise_store_get_trash_folder_sync (CamelStore *store,
 }
 
 static gboolean
-groupwise_can_refresh_folder (CamelStore *store, CamelFolderInfo *info, GError **error)
+groupwise_can_refresh_folder (CamelStore *store,
+                              CamelFolderInfo *info,
+                              GError **error)
 {
-	CamelURL *url;
-	gboolean res;
+	CamelService *service;
+	CamelSettings *settings;
+	gboolean check_all;
 
-	url = camel_service_get_camel_url (CAMEL_SERVICE (store));
+	service = CAMEL_SERVICE (store);
+	settings = camel_service_get_settings (service);
 
-	res = CAMEL_STORE_CLASS (camel_groupwise_store_parent_class)->
-		can_refresh_folder (store, info, error) ||
-		(camel_url_get_param (url, "check_all") != NULL);
+	check_all = camel_groupwise_settings_get_check_all (
+		CAMEL_GROUPWISE_SETTINGS (settings));
 
-	return res;
+	return CAMEL_STORE_CLASS (camel_groupwise_store_parent_class)->
+		can_refresh_folder (store, info, error) || check_all;
 }
 
 /*
@@ -1413,7 +1456,8 @@ camel_groupwise_store_connected (CamelGroupwiseStore *store,
 }
 
 static gint
-match_path (const gchar *path, const gchar *name)
+match_path (const gchar *path,
+            const gchar *name)
 {
 	gchar p, n;
 
@@ -1470,8 +1514,6 @@ groupwise_store_finalize (GObject *object)
 
 	g_free (groupwise_store->priv->user);
 	g_free (groupwise_store->priv->server_name);
-	g_free (groupwise_store->priv->port);
-	g_free (groupwise_store->priv->use_ssl);
 	g_free (groupwise_store->priv->base_url);
 	g_free (groupwise_store->root_container);
 
@@ -1498,7 +1540,6 @@ groupwise_store_initable_init (GInitable *initable,
 	CamelService *service;
 	CamelSession *session;
 	CamelURL *url;
-	const gchar *property_value;
 	const gchar *user_data_dir;
 	const gchar *store_uid;
 	gchar *transport_uid;
@@ -1540,19 +1581,6 @@ groupwise_store_initable_init (GInitable *initable,
 		url, CAMEL_URL_HIDE_PASSWORD |
 		CAMEL_URL_HIDE_PARAMS | CAMEL_URL_HIDE_AUTH);
 
-	/*soap port*/
-	property_value =  camel_url_get_param (url, "soap_port");
-	if (property_value == NULL)
-		groupwise_store->priv->port = g_strdup ("7191");
-	else if (strlen (property_value) == 0)
-		groupwise_store->priv->port = g_strdup ("7191");
-	else
-		groupwise_store->priv->port = g_strdup (property_value);
-
-	/*filter*/
-	if (camel_url_get_param (url, "filter"))
-		CAMEL_STORE (groupwise_store)->flags |= CAMEL_STORE_FILTER_INBOX;
-
 	/*Hash Table*/
 	groupwise_store->priv->id_hash =
 		g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
@@ -1561,10 +1589,6 @@ groupwise_store_initable_init (GInitable *initable,
 	groupwise_store->priv->parent_hash =
 		g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
-	/*ssl*/
-	groupwise_store->priv->use_ssl =
-		g_strdup (camel_url_get_param (url, "use_ssl"));
-
 	CAMEL_STORE (groupwise_store)->flags &= ~CAMEL_STORE_VJUNK;
 	CAMEL_STORE (groupwise_store)->flags &= ~CAMEL_STORE_VTRASH;
 
@@ -1603,6 +1627,7 @@ camel_groupwise_store_class_init (CamelGroupwiseStoreClass *class)
 	object_class->finalize = groupwise_store_finalize;
 
 	service_class = CAMEL_SERVICE_CLASS (class);
+	service_class->settings_type = CAMEL_TYPE_GROUPWISE_SETTINGS;
 	service_class->get_name = groupwise_get_name;
 	service_class->connect_sync = groupwise_connect_sync;
 	service_class->disconnect_sync = groupwise_disconnect_sync;
@@ -1638,8 +1663,6 @@ camel_groupwise_store_init (CamelGroupwiseStore *groupwise_store)
 
 	d("in groupwise store init\n");
 	groupwise_store->priv->server_name = NULL;
-	groupwise_store->priv->port = NULL;
-	groupwise_store->priv->use_ssl = NULL;
 	groupwise_store->priv->user = NULL;
 	groupwise_store->priv->cnc = NULL;
 	groupwise_store->current_folder = NULL;
